/**
 * Copyright (c) 2014 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 * @defgroup pin_change_int_example_main main.c
 * @{
 * @ingroup pin_change_int_example
 * @brief Pin Change Interrupt Example Application main file.
 *
 * This file contains the source code for a sample application using interrupts triggered by GPIO pins.
 *
 */

#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include "nrf.h"
#include "nrf_drv_gpiote.h"
#include "nrf_error.h"
#include "app_error.h"
#include "boards.h"

#include "nrf_drv_ppi.h"
#include "nrf_drv_comp.h"
#include "nrf_delay.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "nrf_drv_i2s.h"
#include "nrf_drv_timer.h"
#include "app_timer.h"

//#include "app_uart.h"
#include "app_fifo.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#define definitions 1

#ifdef definitions
#define DOUT1_PIN				11				/* PIN (P0.11) Check in sdk_config.h */
#define MyID_LENGTH_BYTE 		1				/* ID length in bytes */
#define I2S_BUFFER_SIZE			300				/* I2S buffer size */
#define BORADCAST_TIMEOUT		100				/* Broadcast time out (ms)*/
#define TIMEOUT					500				/* Time out to prevent deadlock */
#define DELAY_TIME				20				/* Radio delay time*/
#define MAX_LED					2				/* Maximum number of LED length */
//W
#define DIN1_PIN				22				/* PIN (P0.22) */

#define MyID_LENGTH_BIT 		MyID_LENGTH_BYTE * 8						/* ID length in bits */
#define TOTAL_NUMBER 			(1 << (MyID_LENGTH_BIT))					/* Total number of nodes*/
#define I2S_DURATION			I2S_BUFFER_SIZE*10*3 + I2S_BUFFER_SIZE*5	/* Transmision time of I2S */
#define I2S_DETECTION			I2S_BUFFER_SIZE*10*3 + I2S_BUFFER_SIZE*5	/* Detection time of I2S */
#define PACKET_SIZE				10											/* size of radioTxBuffer of preprietary radio */
#define UART_TX_BUF_SIZE		256											/* UART TX buffer size */
#define UART_RX_BUF_SIZE		256											/* UART RX buffer size */
static uint32_t					I2STxBuffer[I2S_BUFFER_SIZE];		 		/* I2S tx buffer */
static uint32_t					I2SRxBuffer[I2S_BUFFER_SIZE];				/* I2S rx buffer */
//static uint32_t					dataUART[TOTAL_NUMBER * 3];					/* UART data buffer */
//static uint8_t					dataLED[PACKET_SIZE * PACKET_SIZE];			/* LED data buffer */
bool							flagUART = false;

nrf_drv_i2s_config_t I2S_config = NRF_DRV_I2S_DEFAULT_CONFIG;		/* I2S defaust config */
#endif

#include "app_uart.h"

#define PIN_IN 13

#ifdef BSP_BUTTON_0
    //#define PIN_IN BSP_BUTTON_0
#endif
#ifndef PIN_IN
    #error "Please indicate input pin"
#endif

#ifdef BSP_LED_0
    #define PIN_OUT BSP_LED_0
#endif
#ifndef PIN_OUT
    #error "Please indicate output pin"
#endif

#define DEBUG_PIN 22

#define UART_TX_BUF_SIZE	256								/**< UART TX buffer size. */
#define UART_RX_BUF_SIZE	256								/**< UART RX buffer size. */
#define UART_HWFC			APP_UART_FLOW_CONTROL_DISABLED	/* *Disable UART flow control */

#ifdef APP_UART_ENABLED
void uart_error_handle(app_uart_evt_t * p_event)
{
	if (p_event->evt_type == APP_UART_COMMUNICATION_ERROR)
	{
		APP_ERROR_HANDLER(p_event->data.error_communication);
	}
	else if (p_event->evt_type == APP_UART_FIFO_ERROR)
	{
		APP_ERROR_HANDLER(p_event->data.error_code);
	}
}

/**@brief  Function for initializing the UART module.
 */
 
void uart_init() {
	uint32_t err_code = NRF_SUCCESS;
	const app_uart_comm_params_t comm_params =
	{
		RX_PIN_NUMBER,
		TX_PIN_NUMBER,
		RTS_PIN_NUMBER,
		CTS_PIN_NUMBER,
		UART_HWFC,
		false,
		NRF_UART_BAUDRATE_115200
	};
	APP_UART_FIFO_INIT(&comm_params,
	                   UART_RX_BUF_SIZE,
	                   UART_TX_BUF_SIZE,
	                   uart_error_handle,
	                   APP_IRQ_PRIORITY_LOWEST,
	                   err_code);
	APP_ERROR_CHECK(err_code);
}

#endif


uint32_t changeCount = 0;
bool changed = false;

void in_pin_handler2(){
	changeCount++;
	changed = true;
}

void in_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action){
	//printf("Inside Print Handler!\r\n");
	nrf_drv_gpiote_out_toggle(PIN_OUT);
	changeCount++;
	changed = true;
}
/** @brief Function for configuring: PIN_IN pin for input, PIN_OUT pin for output,
 * and configures GPIOTE to give an interrupt on pin change.
 */

static void gpio_init(void) {
	ret_code_t err_code;

	err_code = nrf_drv_gpiote_init();
	APP_ERROR_CHECK(err_code);

	nrf_drv_gpiote_out_config_t out_config = GPIOTE_CONFIG_OUT_TASK_TOGGLE(false);

	err_code = nrf_drv_gpiote_out_init(PIN_OUT, &out_config);
	APP_ERROR_CHECK(err_code);

	nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(true);
	//nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_HITOLO(true);
	in_config.pull = NRF_GPIO_PIN_PULLUP;
	
	err_code = nrf_drv_gpiote_in_init(PIN_IN, &in_config, in_pin_handler);
	//err_code = nrf_drv_gpiote_in_init(PIN_IN, &in_config, null_handler);
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_event_enable(PIN_IN, true);
	nrf_drv_gpiote_out_task_enable(PIN_OUT);
}

#ifndef DISABLE_COUNTER

nrf_drv_timer_t COUNTER = NRF_DRV_TIMER_INSTANCE(0);

static void counter_handler(nrf_timer_event_t event_type, void * p_context) {
	printf("event_type: %d\r\n", event_type);
}
 
static void counter_init() {
	uint32_t err_code = NRF_SUCCESS;
	
	uint32_t time_ms = 500; //Time(in miliseconds) between consecutive compare events.
    uint32_t time_ticks;

    time_ticks = nrf_drv_timer_ms_to_ticks(&COUNTER, time_ms);
	
	//nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    //err_code = nrf_drv_timer_init(&MY_TIMER, &timer_cfg, timer_event_handler);
	//APP_ERROR_CHECK(err_code);

//    time_ticks = nrf_drv_timer_ms_to_ticks(&COUNTER, time_ms);

//    nrf_drv_timer_extended_compare(&COUNTER, NRF_TIMER_CC_CHANNEL1, time_ticks, NRF_TIMER_SHORT_COMPARE1_CLEAR_MASK, true);
//	
//	err_code = nrf_drv_timer_init(&COUNTER,&timer_cfg,counter_handler);
//	
//    nrf_drv_timer_enable(&COUNTER);

	nrf_drv_timer_config_t counter_conf;
	counter_conf.mode = NRF_TIMER_MODE_COUNTER;
    counter_conf.bit_width = NRF_TIMER_BIT_WIDTH_16;

    
	nrf_drv_timer_extended_compare(&COUNTER, NRF_TIMER_CC_CHANNEL1, time_ticks, NRF_TIMER_SHORT_COMPARE1_CLEAR_MASK, true);
	//APP_ERROR_CHECK();
	err_code = nrf_drv_timer_init(&COUNTER,&counter_conf,counter_handler);
	printf("Error Code: %d\r\n", err_code);
	//COUNTER.p_reg->TASKS_START = 1;
	//nrf_drv_timer_enable(&COUNTER);
}

static void counter_start() {
	nrf_drv_timer_enable(&COUNTER);
//	COUNTER.p_reg->TASKS_CLEAR = 0;
//	COUNTER.p_reg->TASKS_START = 1;
//	COUNTER.p_reg->TASKS_STOP = 0;
}

static void counter_stop() {
//	COUNTER.p_reg->TASKS_START = 0;
//	COUNTER.p_reg->TASKS_STOP = 1;
//	COUNTER.p_reg->TASKS_CLEAR = 1;
	nrf_drv_timer_disable(&COUNTER);
	nrf_drv_timer_clear(&COUNTER);
	//nrf_drv_timer_uninit(&COUNTER);
}

#endif

static nrf_ppi_channel_t m_ppi_channel1;
//#define USETHIS 6
#ifdef USETHIS
// Attach to timer mode
uint32_t capture_frenq() {
	uint32_t frenq_ = nrf_drv_timer_capture(&COUNTER,NRF_TIMER_CC_CHANNEL0);
	COUNTER.p_reg->TASKS_CLEAR = 1;
	COUNTER.p_reg->TASKS_START = 1;
	return frenq_;
}

void timer_void_handler(nrf_timer_event_t event_type, void * p_context) {}

void pin_event_void_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action) {}

//Counter only
void counter_init(void) {
	nrf_drv_timer_config_t conf;
	conf.mode = TIMER_MODE_MODE_Counter;
	conf.bit_width = TIMER_BITMODE_BITMODE_16Bit;
	conf.p_context = &COUNTER;
	printf("Prepare to init counter..\r\n");
	APP_ERROR_CHECK(nrf_drv_timer_init(&COUNTER, &conf, timer_void_handler));
	printf("Enabling counter..\r\n");
	nrf_drv_timer_enable(&COUNTER);
}

void gpiote_init(nrf_drv_gpiote_pin_t frenq_counter_pin) {
	nrf_drv_gpiote_init();
	
	nrf_drv_gpiote_in_config_t config = GPIOTE_CONFIG_IN_SENSE_HITOLO(true);
	config.pull = NRF_GPIO_PIN_PULLUP;
	
	nrf_drv_gpiote_in_init(frenq_counter_pin, &config, pin_event_void_handler);
	nrf_drv_gpiote_in_event_enable(frenq_counter_pin, true);
}

void ppi_init(nrf_drv_gpiote_pin_t frenq_counter_pin){
	uint32_t err_code = NRF_SUCCESS;

	err_code = nrf_drv_ppi_init();
	APP_ERROR_CHECK(err_code);

	err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel1);
	APP_ERROR_CHECK(err_code);
	err_code = nrf_drv_ppi_channel_assign(m_ppi_channel1,
						nrf_drv_gpiote_in_event_addr_get(frenq_counter_pin),
						nrf_drv_timer_task_address_get(&COUNTER, NRF_TIMER_TASK_COUNT));
	APP_ERROR_CHECK(err_code);
	
	nrf_drv_ppi_channel_enable(m_ppi_channel1);
}

void init_it(nrf_drv_gpiote_pin_t pin){
	printf("Init Counter...\r\n");
	counter_init();
	printf("Init GPIOTE...\r\n");
	gpiote_init(pin);
	printf("Init PPI...\r\n");
	ppi_init(pin);
}
#else
static void ppi_init(void) {
    uint32_t err_code = NRF_SUCCESS;
	err_code = nrf_drv_ppi_init();
	APP_ERROR_CHECK(err_code);
    // Configure 1st available PPI channel to stop TIMER0 counter on TIMER1 COMPARE[0] match,
    // which is every even number of seconds.
	
    err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel1);
    APP_ERROR_CHECK(err_code);
	//ppi_enable_channel(FC_PPI_CH, &NRF_GPIOTE->EVENTS_IN[FC_GPIOTE_CH], &FC_TIMER->TASKS_COUNT);
    err_code = nrf_drv_ppi_channel_assign(m_ppi_channel1, \
										  nrf_drv_gpiote_in_event_addr_get(PIN_IN), \
										  nrf_drv_gpiote_out_task_addr_get(PIN_OUT));
										  
	//err_code = nrf_drv_ppi_channel_assign(m_ppi_channel1, \
										  nrf_drv_gpiote_in_event_addr_get(PIN_IN), \
										  nrf_drv_timer_task_address_get(&COUNTER, NRF_TIMER_TASK_CAPTURE1));
    APP_ERROR_CHECK(err_code);

    // Enable both configured PPI channels
    err_code = nrf_drv_ppi_channel_enable(m_ppi_channel1);
    APP_ERROR_CHECK(err_code);
}
#endif

static void pin_init(void) {
	// init LEDs, turn them on
	nrf_gpio_cfg_output(17);
	nrf_gpio_cfg_output(18);
	nrf_gpio_cfg_output(19);
	nrf_gpio_cfg_output(20);

	nrf_delay_ms(500);
	nrf_gpio_pin_set(17);
	nrf_gpio_pin_set(18);
	nrf_gpio_pin_set(19);
	nrf_gpio_pin_set(20);
	
	
	nrf_gpio_cfg_output(DEBUG_PIN);
	nrf_gpio_pin_set(DEBUG_PIN);
	//nrf_delay_us(50);
	nrf_gpio_pin_clear(DEBUG_PIN);
}

#ifdef useComp
/** @brief Comparator's events handler
 * This function is called from interrupt context so it is very important
 * to return quickly. Don't put busy loops or any other CPU intensive actions here.
 * It is also not allowed to call soft device functions from it (if COMP IRQ
 * priority is set to APP_IRQ_PRIORITY_HIGH).
*/

static void comp_event_handler(nrf_comp_event_t event) {
	printf("Inside COMP event handler\r\n");
	/*
	uint32_t value;
    // 0 If the input voltage is below the threshold (VIN+ < VIN-).
    // 1 If the input voltage is above the threshold (VIN+ > VIN-).
    value = nrf_drv_comp_sample();

    if (event == NRF_COMP_EVENT_DOWN)
    {
        g_bEncodermove = true;
        //LOGPRINT(LOG_PRINT_DEBUG, "Event Down - %d",value);
    }
    else if (event == NRF_COMP_EVENT_UP)
    {
        g_bEncodermove = true;
        //LOGPRINT(LOG_PRINT_DEBUG, "Event Up- %d", value);
    }
    else if (event == NRF_COMP_EVENT_READY)
    {
        g_bEncodermove = true;
        //LOGPRINT(LOG_PRINT_DEBUG, "Event Ready");
    }
	*/
	
	#ifdef DEBUG_MODE
	if (event == NRF_COMP_EVENT_DOWN) {
		nrf_gpio_pin_clear(DEBUG_PIN);
	} else {
		nrf_gpio_pin_set(DEBUG_PIN);
	}
	#else
	if (event == NRF_COMP_EVENT_CROSS) {
		nrf_gpio_pin_set(DEBUG_PIN);
		///nrf_delay_us(5);
		nrf_gpio_pin_clear(DEBUG_PIN);
	}
	#endif
	
}

/**
 * @brief Initialize the Comparator, set High and Low Thresholds.
 * Initialize the comparator, and set the High and Low thresholds
*/
static void comp_init(void) {
	printf ("Init COMP\r\n");
	nrf_gpio_pin_clear(17);
	nrf_delay_ms(500);
	nrf_gpio_pin_set(17);
	
	#define lo_volt 0.59 // The voltage indicating "0"
	#define hi_volt 1.4 // The voltage indicating "1"
	double Ref_voltage = 2.4;
	
	double ThresholdLo = (Ref_voltage / 1024) * lo_volt;
	double ThresholdHi = (Ref_voltage / 1024) * hi_volt;
	
	uint32_t err_code = NRF_SUCCESS;
	
	nrf_drv_comp_config_t comp_config = NRF_DRV_COMP_DEFAULT_CONFIG(NRF_COMP_INPUT_0);
	comp_config.isource = NRF_COMP_ISOURCE_Ien5uA;
	// Configure threshold voltages.
	
	comp_config.threshold.th_down = VOLTAGE_THRESHOLD_TO_INT(1.1, 1.2);
    comp_config.threshold.th_up   = VOLTAGE_THRESHOLD_TO_INT(1.3, 1.2);
	
//	comp_config.threshold.th_down = VOLTAGE_THRESHOLD_TO_INT(ThresholdLo, Ref_voltage);
//    comp_config.threshold.th_up   = VOLTAGE_THRESHOLD_TO_INT(ThresholdHi, Ref_voltage);
	
	comp_config.isource = NRF_COMP_ISOURCE_Off;
    comp_config.main_mode = NRF_COMP_MAIN_MODE_SE;
	
	err_code = nrf_drv_comp_init(&comp_config, comp_event_handler);
	
	APP_ERROR_CHECK(err_code);
	nrf_drv_comp_start(NRF_DRV_COMP_EVT_EN_CROSS_MASK, NULL);
}
#endif

uint32_t uartData;

/** @brief Function for transforming LED RGB data to I2S input.
	@param[in] level Value of LED RGB data.
	@param[out] val	Transformed data for I2S input
 */
uint32_t caclChannelValue(uint8_t level) {
	uint32_t val = 0;

	// 0
	if (level == 0) {
		val = 0x88888888;
	}
	// 255
	else if (level == 255) {
		val = 0xeeeeeeee;
	}
	else {
		// apply 4-bit 0xe HIGH pattern wherever level bits are 1.
		val = 0x88888888;
		for (uint8_t i = 0; i < 8; i++) {
			if ((1 << i) & level) {
				uint32_t mask = ~(0x0f << 4 * i);
				uint32_t patt = (0x0e << 4 * i);
				val = (val & mask) | patt;
			}
		}

		// swap 16 bits
		val = (val >> 16) | (val << 16);
	}

	return val;
}

#include <inttypes.h>
void set_led_data(uint8_t category, uint8_t length) {
	switch (category) {
		case 0:
			for (int i = 0; i < 3 * length; i += 3) {
				I2STxBuffer[i] = 0x88888888;
				I2STxBuffer[i+1] = 0x88888888;
				I2STxBuffer[i+2] = 0x88888888;
			 }
			break;
		case 1:
			// for(int i=0; i< 9; i+=3){
			//		I2STxBuffer[i] = 0x88888888;
			//		I2STxBuffer[i+1] = 0x88888888;
			//		I2STxBuffer[i+2] = 0x88888888;
			// }
			for (int i = 0; i < 3 * length; i += 3) {
				I2STxBuffer[i] 	 = caclChannelValue(16);
				I2STxBuffer[i+1] = 0x88888888;
				I2STxBuffer[i+2] = caclChannelValue(16);
			}
			break;
		case 2:
			for (int i = 0; i < 3 * length; i += 3) {
				I2STxBuffer[i] 	 = 0x88888888;
				I2STxBuffer[i+1] = caclChannelValue(16);
				I2STxBuffer[i+2] = caclChannelValue(16);
			}
			break;
		case 3:
			for (int i = 0; i < 3 * length; i += 3) {
				I2STxBuffer[i] 	 = caclChannelValue(16);
				I2STxBuffer[i+1] = caclChannelValue(16);
				I2STxBuffer[i+2] = 0x88888888;
			}
			break;
		}
}

uint32_t my_rx_buf;

/** @brief Callback function for I2S.
	@warning This function is useless in this project, but
			 it should exist. Otherwise, compiler will report
			 error because of function pointer.
*/
static void data_handler(uint32_t const * p_data_received, uint32_t * p_data_to_send, uint16_t number_of_words) {
	if (p_data_received != NULL) {
		//printf("Received a data: %x\r\n", p_data_received[0]);
	}
	if (p_data_to_send != NULL) {
		//printf("Sending data: %x\r\n", p_data_to_send[0]);
	}
}

void i2s_config_init(int DIN_PIN, int DOUT_PIN) {
	uint32_t err_code = NRF_SUCCESS;
	//I2S_config.lrck_pin = 17;
	I2S_config.sdin_pin	 = DIN_PIN;
	I2S_config.sdout_pin = DOUT_PIN;
	I2S_config.mck_setup = NRF_I2S_MCK_32MDIV10;
	I2S_config.ratio	 = NRF_I2S_RATIO_32X;
	I2S_config.channels	 = NRF_I2S_CHANNELS_STEREO;
	err_code = nrf_drv_i2s_init(&I2S_config, data_handler);
	//W
	I2S_config.irq_priority		 = APP_IRQ_PRIORITY_HIGH;
	I2S_config.mode				 = NRF_I2S_MODE_MASTER;
	I2S_config.format			 = NRF_I2S_FORMAT_I2S;
	I2S_config.alignment		 = NRF_I2S_ALIGN_LEFT;
	I2S_config.sample_width		 = NRF_I2S_SWIDTH_16BIT;
	NRF_I2S->RXD.PTR			 = my_rx_buf;


	APP_ERROR_CHECK(err_code);
}

/** @brief Function for starting I2S transmission.
 */
void i2s_start() {
	// Enable reception
	NRF_I2S->CONFIG.RXEN = (I2S_CONFIG_RXEN_RXEN_Enabled << I2S_CONFIG_RXEN_RXEN_Pos);
	// Enable transmission
	NRF_I2S->CONFIG.TXEN = (I2S_CONFIG_TXEN_TXEN_Enabled << I2S_CONFIG_TXEN_TXEN_Pos);
	NRF_I2S->ENABLE = 1;
	nrf_drv_i2s_start(I2SRxBuffer, I2STxBuffer, I2S_BUFFER_SIZE, 0);
}

/** @brief Function for stoping I2S transmission and uninit.
 */
void i2s_stop() {
	nrf_drv_i2s_stop();
	NRF_I2S->ENABLE = 0;
	// Disable reception
	NRF_I2S->CONFIG.RXEN = (I2S_CONFIG_RXEN_RXEN_Disabled << I2S_CONFIG_RXEN_RXEN_Pos);
	// Disable transmission
	NRF_I2S->CONFIG.TXEN = (I2S_CONFIG_TXEN_TXEN_Disabled << I2S_CONFIG_TXEN_TXEN_Pos);

	nrf_drv_i2s_uninit();
}

/** @brief Function for transmission I2S data.
 *	@param[in] color	Color of LED (RED, GREEN, BLUE, or turn OFF)
 *	@param[in] length Numbers of LEDs to light
 *	@param[in] dir		Direction of LED strip light (0 => NORTH, 1 => SOUTH, 2 => WEST, 3 => EAST)
 */
void i2s_data_transmit(uint8_t color, uint32_t length, uint8_t dir) {
	//printf("%d: Sending to: %d\r\n", __LINE__, dir);
	if (dir == 0) {
		i2s_config_init(DIN1_PIN, DOUT1_PIN);
		set_led_data(color, length);
		i2s_start();
		nrf_delay_ms(10);
		i2s_stop();
	}
}

const nrf_drv_timer_t MY_TIMER = NRF_DRV_TIMER_INSTANCE(0);

#define stolen 123

#ifndef stolen

#define FC_GPIO   PIN_IN
#define FC_TIMER  NRF_TIMER0
#define FC_GPIOTE_CH  0
#define FC_PPI_CH  0

static void ppi_enable_channel(uint32_t ch_num, volatile uint32_t *event_ptr, volatile uint32_t *task_ptr) {
	if(ch_num >= 16)
		return;
    else {
		// Otherwise we configure the channel and return the channel number
		NRF_PPI->CH[ch_num].EEP = (uint32_t)event_ptr;
		NRF_PPI->CH[ch_num].TEP = (uint32_t)task_ptr;
		NRF_PPI->CHENSET = (1 << ch_num);
    }
}


// Create a PPI channel: GPIOTE event from low to high --> FC_TIMER count up by 1 
static void counting_init(void) { 
	nrf_gpio_cfg_input(FC_GPIO, NRF_GPIO_PIN_NOPULL);   // frequency counting input

	// Init timer
	FC_TIMER->TASKS_CLEAR = 1;
	FC_TIMER->BITMODE = TIMER_BITMODE_BITMODE_16Bit;
	FC_TIMER->MODE = TIMER_MODE_MODE_Counter;
	FC_TIMER->PRESCALER = 0; 
	FC_TIMER->TASKS_CAPTURE[0] = 0;

	// Init GPIOTE
	//nrf_gpiote_event_config(FC_GPIOTE_CH, FC_GPIO, NRF_GPIOTE_POLARITY_LOTOHI);

	// enable PPI
	ppi_enable_channel(FC_PPI_CH, &NRF_GPIOTE->EVENTS_IN[FC_GPIOTE_CH], &FC_TIMER->TASKS_COUNT);
}

// Destructor 
static void counting_deinit(void)
{
	FC_TIMER->TASKS_STOP = 1;
	FC_TIMER->TASKS_SHUTDOWN = 1;
	//nrf_gpiote_unconfig(FC_GPIOTE_CH);
	nrf_gpio_cfg_output(FC_GPIO);
	nrf_gpio_pin_clear(FC_GPIO);
}

/**
 * @brief The counter
 */

void counter_handler(nrf_timer_event_t event_type, void* p_context) {
	printf("Entered the counter_handler\r\n");
	count_get();
//    switch (event_type) {
//        case NRF_TIMER_EVENT_COMPARE1:
//			in_pin_handler2();	
//            break;
//        default:
//            //Do nothing.
//            break;
//    }
}

#endif

uint32_t captured_t[40];
uint8_t captured_c;
bool captured_f = false;

static void count_get(void) {
	uint32_t got_time = nrf_drv_timer_capture(&COUNTER, NRF_TIMER_CC_CHANNEL1);
	captured_c++;
	counter_stop();
	//printf("TASKS_COUNT: %u\r\n", &COUNTER.p_reg->TASKS_COUNT);
	printf("captured_c: %d\r\n", captured_c);
	printf("count: %d\r\n",	got_time);
	//printf("nrf_timer_cc_read: %d\r\n", nrf_timer_cc_read(COUNTER.p_reg, NRF_TIMER_CC_CHANNEL1));
	counter_start();
//	COUNTER.p_reg->TASKS_CLEAR = 1;
//	COUNTER.p_reg->TASKS_START = 1;
	//nrf_drv_timer_clear(&COUNTER);
	//nrf_drv_timer_enable(&COUNTER);
	
	
}

/**
 * @brief Handler for timer events.
 */
void timer_event_handler(nrf_timer_event_t event_type, void* p_context) {
	//count_get();
}

/**
 * @brief Function for processing UART request
 */
static void process_UART() {
	char UARTAction[10];
	//char stringData[20];
	//uint8_t LEDData[10];

	scanf("%s", UARTAction);
	UARTAction[0] = UARTAction[0] - 48;

	switch (UARTAction[0]) {
	case 0:
	case 1: // Turns on LED
	case 2:
	case 3:
		printf("Turning on LEDs...\r\n");
		i2s_data_transmit(UARTAction[0], MAX_LED, 0); // p1 = color, p2 = length, p3 = direction
		break;
	case 4:
		printf("Calling count_get()...\r\n");
		count_get();
		break;
	case 5:
		
		break;
	default:
		printf("UARTAction[0] = %s\r\n", UARTAction);
	}
}

/**
 * @brief Function for application main entry.
 */
int main(void) {
	uart_init();
	
	uint32_t time_ms = 500; //Time(in miliseconds) between consecutive compare events.
    uint32_t time_ticks;
    uint32_t err_code = NRF_SUCCESS;

	nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    err_code = nrf_drv_timer_init(&MY_TIMER, &timer_cfg, timer_event_handler);
	
	APP_ERROR_CHECK(err_code);
	//printf("%d: error_code: %d\r\n", __LINE__, err_code);
	
    time_ticks = nrf_drv_timer_ms_to_ticks(&MY_TIMER, time_ms);

    nrf_drv_timer_extended_compare(&MY_TIMER, NRF_TIMER_CC_CHANNEL0, time_ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);

    nrf_drv_timer_enable(&MY_TIMER);
	
    //err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);
	
	
	printf("HEllzoI!\r\n");

	gpio_init();
	pin_init();
	//comp_init();
	ppi_init();
	
	//init_it(PIN_IN);
	printf("Init Counter...\r\n");
	counter_init();
	printf("Starting Counter...\r\n");
	counter_start();
	
	printf("Ready...\r\n");
    while (true) {
		//nrf_delay_ms(500);
		if(!flagUART) {
			//printf("count_get: %x\r\n",count_get());
			process_UART();
		}
        // Do nothing.
    }
}


/** @} */
